//----------------------------------------------------------------------------------------
//	File:		Demo3D.cp
//
//	Contains:	3D drawing demo program
//
//	Written by:	Jamie Osborne
// 				Adapted for 3.1.1 by Jeroen Schalk
//
//	Copyright:	 1992-1994 by Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#include "UDemo3D.h"

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>
#endif

#ifndef __UCONTROL__
#include <UControl.h>
#endif

#ifndef __U3DDRAWING__
#include "U3DDrawing.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
const short c3DDrawingWind = 1001;
const short c3DDrawingProcWind = 1002;

const short kModalDialog = 1010;
const short kBadWindow = 1011;

const short kStrings 	= 2000;

//========================================================================================
// CLASS TTestApplication
//========================================================================================

//----------------------------------------------------------------------------------------
// TTestApplication::ITestApplication:
//----------------------------------------------------------------------------------------
#pragma segment AInit

#undef Inherited
#define Inherited TApplication
DefineClass(TTestApplication, TApplication);

void TTestApplication::ITestApplication()
{
	this->IApplication(kFileType, kSignature);

	macroDontDeadStrip(TDemoWindow);

	fLaunchWithNewDocument = false;		// Suppress the creation of a new document at launch

	this->DoMenuCommand(c3DDrawingWind);		// Show the View window

} // TTestApplication::ITestApplication

//----------------------------------------------------------------------------------------
// TTestApplication::DoMakeDocument:
//----------------------------------------------------------------------------------------
#pragma segment AOpen

TDocument* TTestApplication::DoMakeDocument(CommandNumber	/*itsCommandNumber*/,
										   TFile*       	/*itsFile*/ )// override
{
	return NULL;
} // TTestApplication::DoMakeDocument

//----------------------------------------------------------------------------------------
// TTestApplication::DoMenuCommand:
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TTestApplication::DoMenuCommand(CommandNumber aCommandNumber)// override
{
	switch (aCommandNumber)
	{
	case c3DDrawingWind:
		this->MakeDrawingWind();
		break;
	case c3DDrawingProcWind:
		this->MakeDrawingProcWind();
		break;
	default:
		Inherited::DoMenuCommand(aCommandNumber);
		break;
	}
} // TTestApplication::DoMenuCommand

//----------------------------------------------------------------------------------------
// TTestApplication::DoSetupMenus:
//----------------------------------------------------------------------------------------
#pragma segment ARes

void TTestApplication::DoSetupMenus()// override
{
	Inherited::DoSetupMenus();

	Enable(c3DDrawingWind, true);
	Enable(c3DDrawingProcWind, true);

} // TTestApplication::DoSetupMenus

//----------------------------------------------------------------------------------------
// TTestApplication::MakeDrawingWind:
//----------------------------------------------------------------------------------------
#pragma segment ARes

void TTestApplication::MakeDrawingWind()
{
	TWindow *		aWindow = gViewServer->NewTemplateWindow(c3DDrawingWind, NULL);
	FailNIL(aWindow);
	aWindow->Open();
	
} // TTestApplication::MakeTemplateViews

//----------------------------------------------------------------------------------------
// TTestApplication::MakeDrawingProcWind:
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TTestApplication::MakeDrawingProcWind()
{
	TWindow *aWindow;
	VPoint	itsLocation, itsSize;

	FailNIL(aWindow = gViewServer->NewTemplateWindow(c3DDrawingProcWind, NULL));

	TStaticText *aStaticText = new TStaticText;
	itsLocation = VPoint(15, 13);
	itsSize = VPoint(362, 16);
	aStaticText->IStaticText(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, kStrings, 1);

	TStaticText *aStaticText2 = new TStaticText;
	itsLocation = VPoint(278, 188);
	itsSize = VPoint(188, 48);
	aStaticText2->IStaticText(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, kStrings, 2);

	TStaticText *aStaticText3 = new TStaticText;
	itsLocation = VPoint(269, 53);
	itsSize = VPoint(116, 16);
	aStaticText3->IStaticText(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, kStrings, 3);

	TCluster *aCluster = new TCluster;
	itsLocation = VPoint(266, 120);
	itsSize = VPoint(199, 53);
	aCluster->ICluster(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, kNoResource, 0);
	aCluster->SetLabel("A Cluster!", kRedraw);

	itsSize = VPoint(78, 18);
	T3DRadio *aRadio1 = new T3DRadio;
	itsLocation = VPoint(10, 13);
	aRadio1->I3DRadio(aCluster, itsLocation, itsSize, sizeVariable, sizeVariable, "Radio 1", true);

	T3DRadio *aRadio2 = new T3DRadio;
	itsLocation = VPoint(10, 31);
	aRadio2->I3DRadio(aCluster, itsLocation, itsSize, sizeVariable, sizeVariable, "Radio 2" , false);

	T3DRadio *aRadio3 = new T3DRadio;
	itsLocation = VPoint(107, 13);
	aRadio3->I3DRadio(aCluster, itsLocation, itsSize, sizeVariable, sizeVariable, "Disabled", false);

	T3DRadio *aRadio4 = new T3DRadio;
	itsLocation = VPoint(107, 31);
	aRadio4->I3DRadio(aCluster, itsLocation, itsSize, sizeVariable, sizeVariable, "Disabled", false);
	aRadio4->DimState(true, false);
	aRadio4->SetEnable(false);

	T3DCheckBox *aCheckBox = new T3DCheckBox;
	itsLocation = VPoint(26, 203);
	itsSize = VPoint(154, 18);
	aCheckBox->I3DCheckBox(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, "Enabled Check Box", false);

	T3DCheckBox *aCheckBox2 = new T3DCheckBox;
	itsLocation = VPoint(26, 218);
	itsSize = VPoint(154, 28);
	aCheckBox2->I3DCheckBox(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, "Disabled Check Box", false);
	aCheckBox2->DimState(true, false);
	aCheckBox2->SetEnable(false);

	T3DButton *aButton = new T3DButton;
	itsLocation = VPoint(374, 244);
	itsSize = VPoint(82, 20);
	aButton->I3DButton(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, "Show Me");
	aButton->fIdentifier = 'BDBn';
	aButton->SetHilitedTextColor(CRGBColor(65535, 9109, 2210));

	T3DButton *aButton2 = new T3DButton;
	itsLocation = VPoint(280, 244);
	itsSize = VPoint(82, 20);
	aButton2->I3DButton(aWindow, itsLocation, itsSize, sizeVariable, sizeVariable, "Disabled");
	aButton2->DimState(true, false);
	aButton2->SetEnable(false);

	T3DIconButton *anIcon = new T3DIconButton;
	itsLocation = VPoint(36, 76);
	itsSize = VPoint(26, 26);
	anIcon->I3DIconButton(aWindow,itsLocation, itsSize, sizeFixed, sizeFixed, 16, 1002, kButtonMode, false);
	anIcon->fIdentifier = 'IBn1';

	T3DIconButton *anIcon2 = new T3DIconButton;
	itsLocation = VPoint(36, 121);
	itsSize = VPoint(26, 26);
	anIcon2->I3DIconButton(aWindow,itsLocation, itsSize, sizeFixed, sizeFixed, 16, 1002, kButtonMode, false);
	anIcon2->DimState(true, false);
	anIcon2->SetEnable(false);

	TView *aView = new TView;
	itsLocation = VPoint(246, 54);
	itsSize = VPoint(2, 210);
	aView->IView(NULL,aWindow,itsLocation, itsSize, sizeFixed, sizeFixed);

	T3DLineLeftAdorner *aLineAdorner = new T3DLineLeftAdorner;
	aLineAdorner->I3DLineLeftAdorner(true);
	aView->AddAdorner(aLineAdorner, kAdornLast, true);

	TEditText *anEditText = new TEditText;
	itsLocation = VPoint(269, 69);
	itsSize = VPoint(199, 22);
	anEditText->IEditText(aWindow, itsLocation, itsSize, 255);
	anEditText->SetText("Edit Text", kDontRedraw);
	anEditText->DeleteAdornerByID(kFrameAdorner, true);		// We make our own frame

	TWhiteBackgroundAdorner	*back = new TWhiteBackgroundAdorner;
	back->IWhiteBackgroundAdorner(true);

	T3DFrameAdorner	*frame = new T3DFrameAdorner;
	frame->I3DFrameAdorner(true);

	anEditText->AddAdorner(back, kAdornFirst, true);
	anEditText->AddAdorner(frame, kAdornLast, true);

	TControl *aControl = new TControl;
	itsLocation = VPoint(93, 68);
	itsSize = VPoint(134, 90);
	aControl->IControl(aWindow,itsLocation, itsSize, sizeFixed, sizeFixed);

	TWhiteBackgroundAdorner	*back2 = new TWhiteBackgroundAdorner;
	back2->IWhiteBackgroundAdorner(true);

	T3DFrameAdorner	*frame2 = new T3DFrameAdorner;
	frame2->I3DFrameAdorner(true);
	aControl->AddAdorner(back2, kAdornFirst, true);
	aControl->AddAdorner(frame2, kAdornLast, true);

	TextStyle style;
	GetPortTextStyle(style);
	itsLocation = VPoint(2,2);
	itsSize = VPoint(115, 86);
	TScroller *scroller = new TScroller;
	scroller->IScroller(aControl, itsLocation, itsSize, sizeRelSuperView, sizeRelSuperView,
								VPoint(86, 86), false, true);

	TTextListView *aTextListView = new TTextListView;
	aTextListView->ITextListView(NULL, scroller, itsLocation, itsSize, sizeSuperView,
										 sizeSuperView, 2, 16, 0, false, false, 2, 2, true, style);


	aWindow->Open();
	
} // TTestApplication::MakeProcedureViews

//----------------------------------------------------------------------------------------

#undef Inherited
#define Inherited TWindow
DefineClass(TDemoWindow, TWindow);

void TDemoWindow::DoEvent(EventNumber eventNumber,
						 TEventHandler* source,
						 TEvent* event)
{
	switch(eventNumber)
	{
	case mButtonHit:
		if (source == this->FindSubView('BDBn'))
		{
			TWindow *aWindow;
			FailNIL(aWindow = gViewServer->NewTemplateWindow(kBadWindow, NULL));
			aWindow->Open();
		}
		break;
	case mIconButtonHit:
		if (source == this->FindSubView('IBn1'))
		{
			TWindow *aWindow;
			FailNIL(aWindow = gViewServer->NewTemplateWindow(kModalDialog, NULL));
			IDType dismisser = aWindow->PoseModally();
			if (dismisser == 'ok  ')
				aWindow->CloseAndFree();
		}
		break;
	default:
		Inherited::DoEvent(eventNumber, source, event);
		break;
	}
}
