/*
	File:		Orbits.r

	Contains:	Planetary orbit display templates

	Written by:	Harry Chesley

	Copyright:	 1994 by Apple Computer, Inc..

	Change History (most recent first):

		 		  5/25/94	hrc		created

	To Do:
*/

#include "Types.r"
#include "OCE.r"

#include "Orbits.h"

include "Orbits.rsrc" not 'ckid';

#define kOrbitsRecordType "hrc  Orbits"
#define kPlanetAttributeType "hrc  Planet"

#define kOrbitsMainAspect	1000
#define kOrbitsListPage	1100
#define kOrbitsPlotPage	1200

// Card size:

#define kOrbitsPageWidth 400
#define kOrbitsPageHeight 300

// Properties in main aspect:

#define kOrbitsAttributeType	"hrc  Orbits info"

resource 'deta' (kOrbitsMainAspect, purgeable) {
	0,					// Priority (unused)
	dropCheckConflicts,	// Drop check flag
	isMainAspect		// Is the main aspect
};

resource 'rstr' (kOrbitsMainAspect+kDETTemplateName, purgeable) {
	"hrc  Orbits main aspect"
};

resource 'rstr' (kOrbitsMainAspect+kDETRecordType, purgeable) {
	kOrbitsRecordType
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectKind, purgeable) {
	"orbits"
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectWhatIs, purgeable) {
		"Orbits\n\This is a set of solar orbits."
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectAliasKind, purgeable) {
	"orbits alias"
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectAliasWhatIs, purgeable) {
		"Orbits alias\n\nThis is an alias to a set of solar orbits. "
		"To see the orbits, open this icon."
};

resource 'rst#' (kOrbitsMainAspect+kDETAspectCategory, purgeable) {
	{ "Celestial" }
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectNewMenuName, purgeable) {
	"New Orbits"
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectNewEntryName, purgeable) {
	"untitled orbits"
};

// attributes to lookup
resource 'dett' (kOrbitsMainAspect+kDETAspectLookup, purgeable)
	{{
	{ kOrbitsAttributeType }, typeBinary, useForInput, useForOutput, notInSublist, isNotAlias, isNotRecordRef,
		{
		'long', kOrbitsNowProperty,		0;
		};
	{ kPlanetAttributeType }, 0, notForInput, notForOutput, useInSublist, isNotAlias, isNotRecordRef,
		{
		};
	}};

resource 'rst#' (kOrbitsMainAspect+kDETAspectBalloons,purgeable)
	{{
	"The current time.", "The current time. Cannot be edited because the user record is locked or you do not have privileges to make changes here.",
	}};

data 'detb' (kOrbitsMainAspect+kOrbitsTimeProperty, purgeable)
	{ $"0000 0000 0000 0000 0000" };

resource 'detn' (kOrbitsMainAspect+kOrbitsNowProperty, purgeable)
	{ 1 };

resource 'detn' (kOrbitsMainAspect+kOrbitsCustomViewProperty, purgeable)
	{ 0 };

resource 'rstr' (kOrbitsMainAspect+kDETAspectDragInString, purgeable) {
	"Do you want to add %3%^3%the selected items% to the orbits list of *0x/the/* ^1 ^2?"
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectDragInVerb, purgeable) {
	"Add"
};

resource 'rstr' (kOrbitsMainAspect+kDETAspectDragInSummary, purgeable) {
	"Add Body"
};

resource 'rst#' (kOrbitsMainAspect+kDETAspectAttrDragIn,purgeable)
	{{
	"", kPlanetAttributeType, kPlanetAttributeType
	}};

resource 'detw' (kOrbitsMainAspect+kDETAspectInfoPageCustomWindow, purgeable)
	{
	{0,0,kOrbitsPageHeight,kOrbitsPageWidth},
	includePopup
	};

#define kGeneva 3
#define kPalatino 16
#define kTimes	20

#define kDateLineTop	12
#define kDateLineBottom	(kDateLineTop+16)

#define kNowTop			(kDateLineBottom+4)
#define kNowBottom		(kNowTop+16)

#define kTitleTop		(kNowBottom+12)
#define kTitleBottom	(kTitleTop+12)

#define kSublistTop		(kTitleBottom+2)
#define kSublistBottom	(kOrbitsPageHeight-32)
#define kSublistLeft	8
#define kSublistRight	(kOrbitsPageWidth-8)

#define kButtonTop		(kSublistBottom+8)
#define kButtonBottom	(kButtonTop+16)

#define kOpenLeft		62
#define kOpenRight		112

#define kAddLeft		226
#define kAddRight		276

#define kRemoveLeft		288
#define kRemoveRight	338

#define kIconLeft		2
#define kNameLeft		22
#define kKindLeft		116
#define kXLeft			162
#define kYLeft			265
#define kYRight			368

#define kIconEntryTop		-7
#define kIconEntryBottom	9
#define kEntryTop			-5
#define kEntryBottom		9

resource 'deti' (kOrbitsListPage, purgeable) {
	1000,
	{ kSublistTop, kSublistLeft, kSublistBottom, kSublistRight },
	noSelectFirstText,
	{
	kDETNoProperty, kDETNoProperty, kOrbitsListPage;
	},
	{
	kDETNoProperty, kDETNoProperty, kOrbitsListPage+1;
	}};

resource 'rstr' (kOrbitsListPage+kDETTemplateName, purgeable) {
	"hrc  Orbits list page"
};

resource 'rstr' (kOrbitsListPage+kDETRecordType, purgeable) {
	kOrbitsRecordType
};

resource 'rstr' (kOrbitsListPage+kDETInfoPageName, purgeable) {
	"List"
};

resource 'rstr' (kOrbitsListPage+kDETInfoPageMainViewAspect, purgeable) {
	"hrc  Orbits main aspect"
};

// subPage view

resource 'detv' (kOrbitsListPage, purgeable)
	{
	{
	kDETSubpageIconRect, kDETNoFlags, kDETAspectMainBitmap,
		Bitmap { kDETLargeIcon };

	{kDateLineTop, kOrbitsPageWidth-196, kDateLineBottom, kOrbitsPageWidth-160}, kDETNoFlags, kDETNoProperty,
	StaticTextFromView { kPalatino, 10, kDETLeft, kDETBold, "Time" };
	{kDateLineTop-2, kOrbitsPageWidth-160, kDateLineBottom, kOrbitsPageWidth-8}, kDETEnabled, kOrbitsTimeProperty,
	EditText { kPalatino, 12, kDETLeft, kDETNormal };

	{kNowTop, kOrbitsPageWidth-160, kNowBottom, kOrbitsPageWidth-8},
		kDETNoFlags, kOrbitsNowProperty,
		CheckBox { kPalatino, 10, kDETLeft, kDETBold, "Continuous Update", kOrbitsNowProperty };

	{kSublistTop-1, kSublistLeft-1, kSublistBottom+1, kSublistRight+1}, kDETNoFlags, kDETNoProperty,
		Box { kDETUnused };

	{kTitleTop, kSublistLeft+kNameLeft, kTitleBottom, kSublistLeft+kKindLeft-2},
		kDETNoFlags, kDETNoProperty,
		StaticTextFromView { kPalatino, 10, kDETLeft, kDETNormal, "Name" }; 

	{kTitleTop, kSublistLeft+kKindLeft, kTitleBottom, kSublistLeft+kXLeft},
		kDETNoFlags, kDETNoProperty,
		StaticTextFromView { kPalatino, 10, kDETLeft, kDETNormal, "Kind" }; 

	{kTitleTop, kSublistLeft+kXLeft, kTitleBottom, kSublistLeft+kYLeft},
		kDETNoFlags, kDETNoProperty,
		StaticTextFromView { kPalatino, 10, kDETCenter, kDETNormal, "X" }; 

	{kTitleTop, kSublistLeft+kYLeft, kTitleBottom, kSublistLeft+kYRight},
		kDETNoFlags, kDETNoProperty,
		StaticTextFromView { kPalatino, 10, kDETCenter, kDETNormal, "Y" }; 

	{kButtonTop, kOpenLeft, kButtonBottom, kOpenRight}, 
		kDETNoFlags, kDETOpenSelectedItems,
		Button { kGeneva, 10,  kDETCenter, kDETNormal, "Open", kDETOpenSelectedItems };

	{kButtonTop, kAddLeft, kButtonBottom, kAddRight}, 
		kDETNoFlags, kDETAddNewItem,
		Button { kGeneva, 10, kDETCenter, kDETNormal, "Add", kDETAddNewItem };

	{kButtonTop, kRemoveLeft, kButtonBottom, kRemoveRight}, 
		kDETNoFlags, kDETRemoveSelectedItems,
		Button { kGeneva, 10, kDETCenter, kDETNormal, "Remove", kDETRemoveSelectedItems };
	};
	};

resource 'detv' (kOrbitsListPage+1, purgeable)
	{
	{
	{kIconEntryTop, kIconLeft, kIconEntryBottom, kNameLeft-4},
		kDETHilightIfSelected, kDETAspectMainBitmap,
		Bitmap { kDETSmallIcon };

	{kEntryTop, kNameLeft, kEntryBottom, kKindLeft-2},
		kDETHilightIfSelected + kDETDynamicSize, kDETAspectName,
		EditText { kPalatino, 12, kDETForceLeft, kDETNormal };

	{kEntryTop, kKindLeft, kEntryBottom, kXLeft},
		kDETNoFlags, kDETAspectKind,
		StaticText { kPalatino, 12, kDETLeft, kDETNormal };

	{kEntryTop, kXLeft, kEntryBottom, kYLeft},
		kDETNoFlags, kXProperty,
		StaticText { kPalatino, 12, kDETCenter, kDETNormal };

	{kEntryTop, kYLeft, kEntryBottom, kYRight},
		kDETNoFlags, kYProperty,
		StaticText { kPalatino, 12, kDETCenter, kDETNormal };
	};
	};

resource 'deti' (kOrbitsPlotPage, purgeable) {
	2000,
	kDETNoSublistRect,
	noSelectFirstText,
	{
	kDETNoProperty, kDETNoProperty, kOrbitsPlotPage;
	},
	{
	}};

resource 'rstr' (kOrbitsPlotPage+kDETTemplateName, purgeable) {
	"hrc  Orbits plot page"
};

resource 'rstr' (kOrbitsPlotPage+kDETRecordType, purgeable) {
	kOrbitsRecordType
};

resource 'rstr' (kOrbitsPlotPage+kDETInfoPageName, purgeable) {
	"Plot"
};

resource 'rstr' (kOrbitsPlotPage+kDETInfoPageMainViewAspect, purgeable) {
	"hrc  Orbits main aspect"
};

// subPage view

resource 'detv' (kOrbitsPlotPage, purgeable)
	{
	{
	kDETSubpageIconRect, kDETNoFlags, kDETAspectMainBitmap,
	Bitmap { kDETLargeIcon };

	{12, kOrbitsPageWidth-120, 28, kOrbitsPageWidth-8},
		kDETNoFlags, kOrbitsCustomViewProperty,
		CheckBox { kPalatino, 12, kDETLeft, kDETBold, "Show Orbits", kOrbitsCustomViewProperty };

	{44, 8, kOrbitsPageHeight-8, kOrbitsPageWidth-8}, kDETNoFlags, kDETNoProperty,
		Box { kDETUnused };

	{47, 11, kOrbitsPageHeight-11, kOrbitsPageWidth-11}, kDETNoFlags, kOrbitsCustomViewProperty,
		Custom { kDETUnused };
	};
	};
