#
#	FILENAME
#		CustomWriter GX.make
#
#	DESCRIPTION
#		Makefile for the CustomWriter GX driver.
#
#	COPYRIGHT
#		Copyright  1995 Apple Computer, Inc.
#		All rights reserved.
#	
#	Modification history
#		1/14/95 - Dave Hersey -	Created from the shell of a hollowed-out
#								ImageWriter driver.
#

#========== Directories Containing Source and Object Files ==========

ObjDir					=	:Objects:
SrcDir					=	
DestDir					=	

#========== Compiler Options ==========

AsmOptions				=	-case obj
RezOptions				=	-i {RIncludes} -d SystemSevenOrLater=1
COptions				=	-b2 -d applec -i {CIncludes}
LinkOptions				=	-ra =resSysHeap,resPurgeable

DriverName 		= "CustomWriter GX"
DriverCreator 	= 'cuWR'
NewSegID		= 0
OldSegID		= 1

# Compile and assemble statements
{ObjDir}OldApp.c.o  {SrcDir}OldApp.c {SrcDir}CommonDefines.h
	Echo "{DriverName}: Compiling old application"
	C {COptions} -o {ObjDir} {SrcDir}OldApp.c

{ObjDir}OldApp.a.o	 	{SrcDir}OldApp.a
	Echo "{DriverName}: Assembling old app jump table"
	Asm {AsmOptions} {MakeJumpTable} {SrcDir}OldApp.a -o {ObjDir}

{ObjDir}NewApp.c.o	 {SrcDir}NewApp.c {SrcDir}CommonDefines.h
	Echo "{DriverName}: Compiling new application"
	C {COptions} -o {ObjDir} {SrcDir}NewApp.c

{ObjDir}NewApp.a.o	 	{SrcDir}NewApp.a
	Echo "{DriverName}: Assembling new app jump table"
	Asm {AsmOptions} {MakeJumpTable} {SrcDir}NewApp.a -o {ObjDir}

{ObjDir}ChooserSupport.c.o	 {SrcDir}ChooserSupport.c {SrcDir}CommonDefines.h
	Echo "{DriverName}: Compiling Chooser support"
	C {COptions} -o {ObjDir} {SrcDir}ChooserSupport.c

{ObjDir}ChooserSupport.a.o	 	{SrcDir}ChooserSupport.a
	Echo "{DriverName}: Assembling Chooser support"
	Asm {AsmOptions} {MakeJumpTable} {SrcDir}ChooserSupport.a -o {ObjDir}

{ObjDir}CustomBufferingAndIO.c.o  {SrcDir}CustomBufferingAndIO.c {SrcDir}CommonDefines.h
	Echo "{DriverName}: Compiling custom buffering and I/O support"
	C {COptions} -o {ObjDir} {SrcDir}CustomBufferingAndIO.c

{ObjDir}GlobalData.c.o	 {SrcDir}GlobalData.c {SrcDir}CommonDefines.h
	Echo "{DriverName}: Compiling global data support"
	C {COptions} -o {ObjDir} {SrcDir}GlobalData.c


# Link and Rez statements
{DestDir}{DriverName}  {SrcDir}OldApp.r {SrcDir}CommonDefines.h
	Echo "{DriverName}: Rezzing old application"
	Rez {RezOptions} -append -o {DestDir}{DriverName} {SrcDir}OldApp.r
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}
		
{DestDir}{DriverName}  {ObjDir}OldApp.c.o
	Echo "{DriverName}: Linking old application"
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}

{DestDir}{DriverName}  {SrcDir}NewApp.r {SrcDir}CommonDefines.h
	Echo "{DriverName}: Rezzing new application"
	Rez {RezOptions} -append -o {DestDir}{DriverName} {SrcDir}NewApp.r
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}

{DestDir}{DriverName}  						
	{ObjDir}NewApp.a.o							
	{ObjDir}NewApp.c.o							
	{ObjDir}CustomBufferingAndIO.c.o			
	{ObjDir}GlobalData.c.o
	Echo "{DriverName}: linking new API"
	Link										
		{ObjDir}NewApp.a.o						
		{ObjDir}NewApp.c.o						
		{ObjDir}CustomBufferingAndIO.c.o		
		{ObjDir}GlobalData.c.o					
		
		"{Libraries}Interface.o" 				
		
		{LinkOptions}							
		-msg noDup								
		-rt 'pdvr'={NewSegID}				
		-c {DriverCreator}						
		-t 'pdvr'							
		-sg {DriverName}						
		-m SD_JumpTable							
		
		-o {DestDir}{DriverName}
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}

{DestDir}{DriverName}  						
	{ObjDir}OldApp.a.o							
	{ObjDir}OldApp.c.o
	Echo "{DriverName}: linking old API"
	Link										
		{ObjDir}OldApp.a.o						
		{ObjDir}OldApp.c.o						
		
		{LinkOptions}							
		-rt 'pdvr'={OldSegID}				
		-c {DriverCreator}						
		-t 'pdvr'							
		-sg {DriverName}						
		-m SD_JumpTable							
		
		-o {DestDir}{DriverName}
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}

{DestDir}{DriverName}  {SrcDir}ChooserSupport.r {SrcDir}CommonDefines.h
	Echo "{DriverName}: Rezzing Chooser resources"
	Rez {RezOptions} -append -o {DestDir}{DriverName} {SrcDir}ChooserSupport.r
	SetFile -a Bi -t 'pdvr' -c {DriverCreator} {DestDir}{DriverName}

{DestDir}{DriverName}  						
	{ObjDir}ChooserSupport.a.o					
	{ObjDir}ChooserSupport.c.o
	Echo "{DriverName}: Linking PACK"
	Link										
		{ObjDir}ChooserSupport.a.o				
		{ObjDir}ChooserSupport.c.o				
		
		-rt PACK=-4096							
		-sg PrintingPACK						
		-m EntryPoint							
		
		-o {Targ}

{DestDir}{DriverName}	 {ObjDir}ChooserSupport.c.o
	Echo "{DriverName}: Linking LDEF"
	Link -w -rt LDEF=-4096 -m LDEF -sg LDEF -o {Targ} {ObjDir}ChooserSupport.c.o
